@@ -55,7 +55,7 @@ enum screenparts {
 	selstatesel,
 	colheads,
 	query,
-	info,
+	info_body,
 	info_head,
 	whatinfo,
 	helpscreen,
@@ -63,8 +63,8 @@ enum screenparts {
 };
 
 struct column {
-	column(): title(nullptr), x(0), width(0) {};
-	void blank() { title = nullptr; x = 0; width = 0; };
+	column(): title(nullptr), x(0), width(0) {}
+	void blank() { title = nullptr; x = 0; width = 0; }
 
 	const char *title;
 	int x;
@@ -87,9 +87,9 @@ protected:
 
   void add_column(column &col, const char *title, int width);
   void end_column(column &col, const char *title);
-  void draw_column_head(column &col);
-  void draw_column_sep(column &col, int y);
-  void draw_column_item(column &col, int y, const char *item);
+  void draw_column_head(const column &col);
+  void draw_column_sep(const column &col, int y);
+  void draw_column_item(const column &col, int y, const char *item);
 
   // (n)curses stuff
   WINDOW *listpad, *infopad, *colheadspad, *thisstatepad;
@@ -97,12 +97,8 @@ protected:
   // If listpad is null, then we have not started to display yet, and
   // so none of the auto-displaying update routines need to display.
 
-  // SIGWINCH handling
-  void sigwinch_mask(int how);
-  void setupsigwinch();
-
-  static baselist *signallist;
-  static void sigwinchhandler(int);
+  // Window resize handling (via SIGWINCH).
+  void resize_window();
 
   int nitems, ldrawnstart, ldrawnend, showinfo;
   int topofscreen, leftofscreen, cursorline;
@@ -167,7 +163,7 @@ public:
   void startdisplay();
   void enddisplay();
 
-  baselist(keybindings *);
+  explicit baselist(keybindings *);
   virtual ~baselist();
 };
 
@@ -190,8 +186,16 @@ extern colordata color[];
 /* Evil recommends flag variable. */
 extern bool manual_install;
 
-enum urqresult { urqr_normal, urqr_fail, urqr_quitmenu };
-enum quitaction { qa_noquit, qa_quitchecksave, qa_quitnochecksave };
+enum urqresult {
+	urqr_normal,
+	urqr_fail,
+	urqr_quitmenu,
+};
+enum quitaction {
+	qa_noquit,
+	qa_quitchecksave,
+	qa_quitnochecksave,
+};
 
 typedef urqresult urqfunction(void);
 urqfunction urq_list, urq_quit, urq_menu;
