@@ -54,6 +54,7 @@
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/options.h>
+#include <dpkg/fsys.h>
 
 #include "dselect.h"
 #include "bindings.h"
@@ -63,8 +64,6 @@ static const char printforhelp[] = N_("Type dselect --help for help.");
 
 bool expertmode = false;
 
-static const char *admindir = ADMINDIR;
-
 static keybindings packagelistbindings(packagelist_kinterps,packagelist_korgbindings);
 
 struct table_t {
@@ -72,7 +71,7 @@ struct table_t {
   const int num;
 };
 
-static const struct table_t colourtable[]= {
+static const struct table_t colortable[] = {
   {"black",	COLOR_BLACK	},
   {"red",	COLOR_RED	},
   {"green",	COLOR_GREEN	},
@@ -114,7 +113,7 @@ static const struct table_t screenparttable[]= {
   {nullptr, 0},
 };
 
-/* Historical (patriotic?) colours. */
+/* Original colors. */
 struct colordata color[]= {
   /* fore      back            attr */
   {COLOR_WHITE,        COLOR_BLACK,    0			}, // default, not used
@@ -187,11 +186,13 @@ usage(const struct cmdinfo *ci, const char *value)
   printf(_(
 "Options:\n"
 "      --admindir <directory>       Use <directory> instead of %s.\n"
+"      --instdir <directory>        Use <directory> instead of %s.\n"
+"      --root <directory>           Use <directory> instead of %s.\n"
 "      --expert                     Turn on expert mode.\n"
 "  -D, --debug <file>               Turn on debugging, send output to <file>.\n"
 "      --color <color-spec>         Configure screen colors.\n"
 "      --colour <color-spec>        Ditto.\n"
-), ADMINDIR);
+), ADMINDIR, "/", "/");
 
   printf(_(
 "  -?, --help                       Show this help message.\n"
@@ -206,8 +207,8 @@ usage(const struct cmdinfo *ci, const char *value)
   fputs("\n", stdout);
 
   printf(_("<color> is:"));
-  for (i=0; colourtable[i].name; i++)
-    printf(" %s", colourtable[i].name);
+  for (i = 0; colortable[i].name; i++)
+    printf(" %s", colortable[i].name);
   fputs("\n", stdout);
 
   printf(_("<attr> is:"));
@@ -266,38 +267,42 @@ extern "C" {
   set_color(const struct cmdinfo*, const char *string)
   {
     char *s;
-    char *colours, *attributes, *attrib, *colourname;
-    int screenpart, aval;
+    char *colors, *attributes;
+    int screenpart;
 
     s = m_strdup(string); // strtok modifies strings, keep string const
     screenpart= findintable(screenparttable, strtok(s, ":"), _("screen part"));
-    colours = strtok(nullptr, ":");
+    colors = strtok(nullptr, ":");
     attributes = strtok(nullptr, ":");
 
-    if ((colours == nullptr || ! strlen(colours)) &&
+    if ((colors == nullptr || ! strlen(colors)) &&
         (attributes == nullptr || ! strlen(attributes))) {
-       ohshit(_("null colour specification"));
+       ohshit(_("missing color specification"));
     }
 
-    if (colours != nullptr && strlen(colours)) {
-      colourname= strtok(colours, ",");
-      if (colourname != nullptr && strlen(colourname)) {
+    if (colors != nullptr && strlen(colors)) {
+      char *colorname;
+
+      colorname = strtok(colors, ",");
+      if (colorname != nullptr && strlen(colorname)) {
         // normalize attributes to prevent confusion
         color[screenpart].attr= A_NORMAL;
-       color[screenpart].fore=findintable(colourtable, colourname, _("colour"));
+        color[screenpart].fore = findintable(colortable, colorname, _("color"));
       }
-      colourname = strtok(nullptr, ",");
-      if (colourname != nullptr && strlen(colourname)) {
+      colorname = strtok(nullptr, ",");
+      if (colorname != nullptr && strlen(colorname)) {
         color[screenpart].attr= A_NORMAL;
-        color[screenpart].back=findintable(colourtable, colourname, _("colour"));
+        color[screenpart].back = findintable(colortable, colorname, _("color"));
       }
     }
 
     if (attributes != nullptr && strlen(attributes)) {
-      for (attrib= strtok(attributes, "+");
+      for (char *attrib = strtok(attributes, "+");
            attrib != nullptr && strlen(attrib);
            attrib = strtok(nullptr, "+")) {
-               aval=findintable(attrtable, attrib, _("colour attribute"));
+               int aval;
+
+               aval = findintable(attrtable, attrib, _("color attribute"));
                if (aval == A_NORMAL) // set to normal
                        color[screenpart].attr= aval;
                else // add to existing attribs
@@ -311,7 +316,9 @@ extern "C" {
 } /* End of extern "C" */
 
 static const struct cmdinfo cmdinfos[]= {
-  { "admindir",     0,  1, nullptr,  &admindir, nullptr      },
+  { "admindir",     0,  1, nullptr,  nullptr,   set_admindir, 1 },
+  { "instdir",      0,  1, nullptr,  nullptr,   set_instdir, 1 },
+  { "root",         0,  1, nullptr,  nullptr,   set_root, 1  },
   { "debug",       'D', 1, nullptr,  nullptr,   set_debug    },
   { "expert",      'E', 0, nullptr,  nullptr,   set_expert   },
   { "help",        '?', 0, nullptr,  nullptr,   usage        },
@@ -354,8 +361,8 @@ void cursesoff() {
 }
 
 urqresult urq_list(void) {
-  modstatdb_open((modstatdb_rw)(msdbrw_writeifposs |
-                                msdbrw_available_readonly));
+  modstatdb_open(static_cast<modstatdb_rw>(msdbrw_writeifposs |
+                                           msdbrw_available_readonly));
 
   curseson();
 
@@ -370,7 +377,7 @@ urqresult urq_list(void) {
 }
 
 static void
-dme(int i, int so)
+display_menu_entry(int i, int so)
 {
   const menuentry *me= &menuentries[i];
 
@@ -406,7 +413,7 @@ refreshmenu(void)
   attrset(A_NORMAL);
   const struct menuentry *mep; int i;
   for (mep=menuentries, i=0; mep->option && mep->menuent; mep++, i++)
-    dme(i,0);
+    display_menu_entry(i, 0);
 
   attrset(A_BOLD);
   addstr(_("\n\n"
@@ -431,25 +438,39 @@ urqresult urq_menu(void) {
   int entries, c;
   entries= refreshmenu();
   int cursor=0;
-  dme(0,1);
+  display_menu_entry(0, 1);
   for (;;) {
     refresh();
-    do
+    do {
       c= getch();
-    while (c == ERR && errno == EINTR);
+      if (c == KEY_RESIZE) {
+        refreshmenu();
+        display_menu_entry(cursor, 1);
+        continue;
+      }
+    } while (c == ERR && errno == EINTR);
     if (c==ERR)  {
       if(errno != 0)
         ohshite(_("failed to getch in main menu"));
       else {
-        clearok(stdscr,TRUE); clear(); refreshmenu(); dme(cursor,1);
+        clearok(stdscr, TRUE);
+        clear();
+        refreshmenu();
+        display_menu_entry(cursor, 1);
       }
     }
 
     if (c == CTRL('n') || c == KEY_DOWN || c == ' ' || c == 'j') {
-      dme(cursor,0); cursor++; cursor %= entries; dme(cursor,1);
+      display_menu_entry(cursor, 0);
+      cursor++;
+      cursor %= entries;
+      display_menu_entry(cursor, 1);
     } else if (c == CTRL('p') || c == KEY_UP || c == CTRL('h') ||
                c==KEY_BACKSPACE || c==KEY_DC || c=='k') {
-      dme(cursor,0); cursor+= entries-1; cursor %= entries; dme(cursor,1);
+      display_menu_entry(cursor, 0);
+      cursor += entries - 1;
+      cursor %= entries;
+      display_menu_entry(cursor, 1);
     } else if (c=='\n' || c=='\r' || c==KEY_ENTER) {
       clear(); refresh();
 
@@ -465,13 +486,19 @@ urqresult urq_menu(void) {
       default:
         internerr("unknown menufn %d", res);
       }
-      refreshmenu(); dme(cursor,1);
+      refreshmenu();
+      display_menu_entry(cursor, 1);
     } else if (c == CTRL('l')) {
-      clearok(stdscr,TRUE); clear(); refreshmenu(); dme(cursor,1);
+      clearok(stdscr, TRUE);
+      clear();
+      refreshmenu();
+      display_menu_entry(cursor, 1);
     } else if (isdigit(c)) {
       char buf[2]; buf[0]=c; buf[1]=0; c=atoi(buf);
       if (c < entries) {
-        dme(cursor,0); cursor=c; dme(cursor,1);
+        display_menu_entry(cursor, 0);
+        cursor = c;
+        display_menu_entry(cursor, 1);
       } else {
         beep();
       }
@@ -481,7 +508,9 @@ urqresult urq_menu(void) {
       while (i < entries && gettext(menuentries[i].key)[0] != c)
         i++;
       if (i < entries) {
-        dme(cursor,0); cursor=i; dme(cursor,1);
+        display_menu_entry(cursor, 0);
+        cursor = i;
+        display_menu_entry(cursor, 1);
       } else {
         beep();
       }
@@ -514,7 +543,7 @@ main(int, const char *const *argv)
   dpkg_options_load(DSELECT, cmdinfos);
   dpkg_options_parse(&argv, cmdinfos, printforhelp);
 
-  admindir = dpkg_db_set_dir(admindir);
+  debug(dbg_general, "root=%s admindir=%s", dpkg_fsys_get_dir(), dpkg_db_get_dir());
 
   if (*argv) {
     const char *a;
@@ -532,6 +561,7 @@ main(int, const char *const *argv)
 
   cursesoff();
   dpkg_program_done();
+  dpkg_locales_done();
 
   return(0);
 }
