@@ -1,12 +1,13 @@
 #!/usr/bin/perl
 #
-# Copyright © 1996 Andy Guy <awpguy@acs.ucalgary.ca>
+# Copyright © 1996 Andy Guy <andy@cyteen.org>
 # Copyright © 1998 Martin Schulze <joey@infodrom.north.de>
 # Copyright © 1999, 2009 Raphaël Hertzog <hertzog@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,15 +21,15 @@ use strict;
 use warnings;
 
 eval q{
-    pop @INC if $INC[-1] eq '.';
-    use Net::FTP;
+    use Dpkg; # Dummy import to require the presence of Dpkg::*.
 };
 if ($@) {
-    warn "Please install the 'perl' package if you want to use the\n" .
-         "FTP access method of dselect.\n\n";
+    warn "Missing Dpkg modules required by the FTP access method.\n\n";
     exit 1;
 }
-use Dselect::Ftp;
+
+use Dselect::Method;
+use Dselect::Method::Ftp;
 
 # deal with arguments
 my $vardir = $ARGV[0];
@@ -43,7 +44,7 @@ if ($option eq 'manual') {
 
 #Defaults
 my $arch = qx(dpkg --print-architecture);
-$arch='i386' if $?;
+$arch = 'i386' if $?;
 chomp $arch;
 
 my $logname = qx(whoami);
@@ -102,14 +103,13 @@ EOM
 
 if (! $CONFIG{done}) {
   view_mirrors() if (yesno('y', 'Would you like to see a list of ftp mirrors'));
-  add_site();
+  add_site('ftp');
 }
-edit_config($methdir);
+edit_config('ftp', $methdir);
 
 my $ftp;
 sub download() {
  foreach (@{$CONFIG{site}}) {
-
     $ftp = do_connect(ftpsite => $_->[0],
                       ftpdir => $_->[1],
                       passive => $_->[3],
@@ -131,13 +131,13 @@ sub download() {
 
 	foreach my $line (@dirlst) {
 	    if($line =~ /Packages/) {
-		$got_pkgfile=1;
+		$got_pkgfile = 1;
 	    }
 	}
 	if( !$got_pkgfile) {
 	    print "Warning: Could not find a Packages file in $dir\n",
 	    "This may not be a problem if the directory is a symbolic link\n";
-	    $problem=1;
+	    $problem = 1;
 	}
     }
     print "Closing ftp connection...\n";
