@@ -151,8 +151,10 @@ void packagelist::addheading(enum ssavailval ssavail,
 static packagelist *sortpackagelist;
 
 int qsort_compareentries(const void *a, const void *b) {
-  return sortpackagelist->compareentries(*(const struct perpackagestate **)a,
-                                         *(const struct perpackagestate **)b);
+  const struct perpackagestate *pa = *static_cast<const struct perpackagestate * const *>(a);
+  const struct perpackagestate *pb = *static_cast<const struct perpackagestate * const *>(b);
+
+  return sortpackagelist->compareentries(pa, pb);
 }
 
 void packagelist::sortinplace() {
@@ -293,7 +295,7 @@ void packagelist::sortmakeheads() {
   int a;
   for (a=0; a<nrealitems; a++) {
     thispkg= table[a]->pkg;
-    if (thispkg->set->name == NULL)
+    if (thispkg->set->name == nullptr)
       internerr("package set has no name at table index %d", a);
     int ssdiff= 0;
     ssavailval ssavail= ssa_none;
@@ -579,11 +581,9 @@ pkginfo **packagelist::display() {
   // returns 0 if no recursive check is desired.
   int response, index;
   const keybindings::interpretation *interp;
-  pkginfo **retl;
 
   debug(dbg_general, "packagelist[%p]::display()", this);
 
-  setupsigwinch();
   startdisplay();
 
   if (!expertmode)
@@ -594,22 +594,23 @@ pkginfo **packagelist::display() {
     if (whatinfo_height) wcursyncup(whatinfowin);
     if (doupdate() == ERR)
       ohshite(_("doupdate failed"));
-    signallist= this;
-    sigwinch_mask(SIG_UNBLOCK);
-    do
-    response= getch();
-    while (response == ERR && errno == EINTR);
-    sigwinch_mask(SIG_BLOCK);
+    do {
+      response = getch();
+      if (response == KEY_RESIZE) {
+        resize_window();
+        continue;
+      }
+    } while (response == ERR && errno == EINTR);
     if (response == ERR)
       ohshite(_("getch failed"));
     interp= (*bindings)(response);
     debug(dbg_general, "packagelist[%p]::display() response=%d interp=%s",
           this, response, interp ? interp->action : "[none]");
-    if (!interp) { beep(); continue; }
+    if (!interp)
+      continue;
     (this->*(interp->pfn))();
     if (interp->qa != qa_noquit) break;
   }
-  pop_cleanup(ehflag_normaltidy); // unset the SIGWINCH handler
   enddisplay();
 
   if (interp->qa == qa_quitnochecksave ||
@@ -619,7 +620,7 @@ pkginfo **packagelist::display() {
   }
 
   if (recursive) {
-    retl= new pkginfo*[nitems+1];
+    pkginfo **retl = new pkginfo*[nitems + 1];
     for (index=0; index<nitems; index++) retl[index]= table[index]->pkg;
     retl[nitems] = nullptr;
     debug(dbg_general, "packagelist[%p]::display() done, retl=%p", this, retl);
