@@ -1,6 +1,6 @@
 /*
  * libdpkg - Debian packaging suite library routines
- * path-remove.c - path removal functionss
+ * path-remove.c - path removal functions
  *
  * Copyright © 1994-1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2007-2015 Guillem Jover <guillem@debian.org>
@@ -24,8 +24,8 @@
 
 #include <sys/stat.h>
 
-#include <assert.h>
 #include <errno.h>
+#include <string.h>
 #include <unistd.h>
 
 #include <dpkg/i18n.h>
@@ -37,9 +37,11 @@
 int
 secure_unlink_statted(const char *pathname, const struct stat *stab)
 {
-	if (S_ISREG(stab->st_mode) ? (stab->st_mode & 07000) :
-	    !(S_ISLNK(stab->st_mode) || S_ISDIR(stab->st_mode) ||
-	      S_ISFIFO(stab->st_mode) || S_ISSOCK(stab->st_mode))) {
+	mode_t mode = stab->st_mode;
+
+	if (S_ISREG(mode) ? (mode & 07000) :
+	    !(S_ISLNK(mode) || S_ISDIR(mode) ||
+	      S_ISFIFO(mode) || S_ISSOCK(mode))) {
 		if (chmod(pathname, 0600))
 			return -1;
 	}
@@ -121,7 +123,8 @@ path_remove_tree(const char *pathname)
 	const char *u;
 
 	u = path_skip_slash_dotslash(pathname);
-	assert(*u);
+	if (u[0] == '\0')
+		internerr("pathname '%s' reduces to nothing", pathname);
 
 	debug(dbg_eachfile, "%s '%s'", __func__, pathname);
 	if (!rmdir(pathname))
@@ -136,6 +139,13 @@ path_remove_tree(const char *pathname)
 		if (errno == ENOTDIR)
 			return;
 	}
+	/* Trying to remove a directory or a file on a read-only filesystem,
+	 * even if non-existent, always returns EROFS. */
+	if (errno == EROFS) {
+		if (access(pathname, F_OK) < 0 && errno == ENOENT)
+			return;
+		errno = EROFS;
+	}
 	if (errno != ENOTEMPTY && errno != EEXIST) /* Huh? */
 		ohshite(_("unable to securely remove '%.255s'"), pathname);
 
