@@ -96,20 +96,12 @@ sub run_hook {
             $fields->{'Launchpad-Bugs-Fixed'} = join(' ', @$bugs);
         }
     } elsif ($hook eq 'update-buildopts') {
-       my $build_opts = shift @params;
-       require Dpkg::Arch;
-       my $arch = Dpkg::Arch::get_host_arch();
-       if (Dpkg::Arch::debarch_eq($arch, 'riscv64')) {
-           $build_opts->set('nocheck', 1, 'riscv64');
-       }
-    } elsif ($hook eq 'update-buildflags') {
-	my $flags = shift @params;
-
-        # Run the Debian hook to add hardening flags
-        $self->SUPER::run_hook($hook, $flags);
-
-	# Per https://wiki.ubuntu.com/DistCompilerFlags
-        $flags->prepend('LDFLAGS', '-Wl,-Bsymbolic-functions');
+        my $build_opts = shift @params;
+        require Dpkg::Arch;
+        my $arch = Dpkg::Arch::get_host_arch();
+        if (Dpkg::Arch::debarch_eq($arch, 'riscv64')) {
+            $build_opts->set('nocheck', 1, 'riscv64');
+        }
     } elsif ($hook eq 'update-buildprofiles') {
         my $build_profiles_ref = shift @params;
         unless(grep $_ =~ /^!?noudeb$/, @$build_profiles_ref) {
@@ -177,6 +169,10 @@ sub init_build_features {
         	$use_feature->{optimize}{lto} = 0;
 	}
     }
+
+    if (any { $_ eq $arch } qw(amd64 arm64 riscv64 s390x)) {
+        $use_feature->{qa}{framepointer} = 1;
+    }
 }
 
 sub set_build_features {
@@ -190,13 +186,21 @@ sub set_build_features {
     if ($arch eq 'ppc64el' && $flags->get_option_value('optimize-level') != 0) {
         $flags->set_option_value('optimize-level', 3);
     }
+
+    $flags->set_option_value('fortify-level', 3);
+
+    # Debian enables -fstack-clash-protection but it causes troubles on armhf
+    # (which are not fully analyzed as of writing this); disable it there in
+    # order to have time to investigate
+    if ($arch eq 'armhf') {
+        # Stack clash protector only available on amd64 and arm.
+        $flags->set_feature('hardening', 'stackclash', 0);
+    }
 }
 
-sub _add_build_flags {
+sub add_build_flags {
     my ($self, $flags) = @_;
 
-    $self->SUPER::_add_build_flags($flags);
-
     my @compile_flags = qw(
         CFLAGS
         CXXFLAGS
@@ -204,9 +208,64 @@ sub _add_build_flags {
         OBJCXXFLAGS
         FFLAGS
         FCFLAGS
-        GCJFLAGS
     );
 
+    $self->SUPER::add_build_flags($flags);
+
+    # Per https://wiki.ubuntu.com/DistCompilerFlags
+    $flags->prepend('LDFLAGS', '-Wl,-Bsymbolic-functions');
+
+    # Add Rust flags
+    $flags->append('RUSTFLAGS', '');
+    $flags->append('RUSTFLAGS_FOR_BUILD', '');
+    # Rust frame pointer switch
+    if ($flags->use_feature('qa', 'framepointer')) {
+        my $arch = Dpkg::Arch::get_host_arch();
+        if (Dpkg::Arch::debarch_eq($arch, 's390x')) {
+            $flags->append('RUSTFLAGS', '-Ctarget-feature=+backchain');
+        } else {
+            $flags->append('RUSTFLAGS', '-Cforce-frame-pointers=yes');
+        }
+    }
+
+    # In Ubuntu these flags are set by the compiler, so when disabling the
+    # features we need to pass appropriate flags to disable them.
+    if (!$flags->use_feature('hardening', 'stackprotectorstrong') &&
+        !$flags->use_feature('hardening', 'stackprotector')) {
+        my $flag = '-fno-stack-protector';
+        $flags->append($_, $flag) foreach @compile_flags;
+    }
+
+    if (!$flags->use_feature('hardening', 'stackclash')) {
+        my $flag = '-fno-stack-clash-protection';
+        $flags->append($_, $flag) foreach @compile_flags;
+    }
+
+    if (!$flags->use_feature('hardening', 'fortify')) {
+        $flags->append('CPPFLAGS', '-D_FORTIFY_SOURCE=0');
+    }
+
+    if (!$flags->use_feature('hardening', 'format')) {
+        my $flag = '-Wno-format -Wno-error=format-security';
+        $flags->append('CFLAGS', $flag);
+        $flags->append('CXXFLAGS', $flag);
+        $flags->append('OBJCFLAGS', $flag);
+        $flags->append('OBJCXXFLAGS', $flag);
+     }
+
+     if (!$flags->use_feature('hardening', 'branch')) {
+        my $cpu = $flags->get_option_value('hardening-branch-cpu');
+        my $flag;
+        if ($cpu eq 'arm64') {
+            $flag = '-mbranch-protection=none';
+        } elsif ($cpu eq 'amd64') {
+            $flag = '-fcf-protection=none';
+        }
+        if (defined $flag) {
+            $flags->append($_, $flag) foreach @compile_flags;
+        }
+    }
+
     # We always enable fdebug-prefix-map and (ideally) set the new
     # path as full (unless ${DEB_BUILD_DEBUGPATH} points to a relative
     # path), because (a) the DWARF spec forbids relative paths (which
@@ -239,6 +298,8 @@ sub _add_build_flags {
             }
         }
     }
+
+    return;
 }
 
 =head1 PUBLIC FUNCTIONS
