@@ -26,12 +26,14 @@ use warnings;
 use File::Temp qw(tempdir);
 use File::Basename;
 use File::Copy;
+use File::Glob qw(bsd_glob GLOB_TILDE GLOB_NOCHECK);
 use POSIX qw(:sys_wait_h);
 
 use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
-use Dpkg::Build::Types;
+use Dpkg::BuildTypes;
+use Dpkg::BuildAPI qw(get_build_api);
 use Dpkg::BuildOptions;
 use Dpkg::BuildProfiles qw(set_build_profiles);
 use Dpkg::Conf;
@@ -42,6 +44,9 @@ use Dpkg::Version;
 use Dpkg::Control;
 use Dpkg::Control::Info;
 use Dpkg::Changelog::Parse;
+use Dpkg::OpenPGP;
+use Dpkg::OpenPGP::ErrorCodes;
+use Dpkg::OpenPGP::KeyHandle;
 use Dpkg::Path qw(find_command);
 use Dpkg::IPC;
 use Dpkg::Vendor qw(run_vendor_hook);
@@ -86,27 +91,34 @@ sub usage {
   -R, --rules-file=<rules>    rules file to execute (default is debian/rules).
   -T, --rules-target=<target> call debian/rules <target>.
       --as-root               ensure -T calls the target with root rights.
-  -j, --jobs[=<number>|auto]  jobs to run simultaneously (passed to <rules>),
-                                forced mode.
-  -J, --jobs-try[=<number>|auto]
+  -j, --jobs[=<jobs>|auto]    jobs to run simultaneously (passed to <rules>),
+                                (default; default is auto, opt-in mode).
+  -J, --jobs-try[=<jobs>|auto]
+                              alias for -j, --jobs.
+      --jobs-force[=<jobs>|auto]
                               jobs to run simultaneously (passed to <rules>),
-                                opt-in mode (default is auto).
+                                (default is auto, forced mode).
   -r, --root-command=<command>
                               command to gain root rights (default is fakeroot).
       --check-command=<command>
                               command to check the .changes file (no default).
       --check-option=<opt>    pass <opt> to check <command>.
       --hook-<name>=<command> set <command> as the hook <name>, known hooks:
-                                init preclean source build binary buildinfo
-                                changes postclean check sign done
+                                preinit init preclean source build binary
+                                buildinfo changes postclean check sign done
       --buildinfo-file=<file> set the .buildinfo filename to generate.
       --buildinfo-option=<opt>
                               pass option <opt> to dpkg-genbuildinfo.
       --changes-file=<file>   set the .changes filename to generate.
+      --sign-backend=<backend>
+                              OpenPGP backend to use to sign
+                                (default is auto).
   -p, --sign-command=<command>
                               command to sign .dsc and/or .changes files
                                 (default is gpg).
-  -k, --sign-key=<keyid>      the key to use for signing.
+      --sign-keyfile=<file>   the key file to use for signing.
+  -k, --sign-keyid=<keyid>    the key id to use for signing.
+      --sign-key=<keyid>      alias for -k, --sign-keyid.
   -ap, --sign-pause           add pause before starting signature process.
   -us, --unsigned-source      unsigned source package.
   -ui, --unsigned-buildinfo   unsigned .buildinfo file.
@@ -128,8 +140,10 @@ sub usage {
   -sa                         source includes orig, always.
   -sd                         source is diff and .dsc only.
   -v<version>                 changes since version <version>.
-  -m, --release-by=<maint>    maintainer for this release is <maint>.
-  -e, --build-by=<maint>      maintainer for this build is <maint>.
+  -m, --source-by=<maint>     maintainer for this source or build is <maint>.
+      --build-by=<maint>      ditto.
+  -e, --release-by=<maint>    maintainer for this change or release is <maint>.
+      --changed-by=<maint>    ditto.
   -C<descfile>                changes are described in <descfile>.
       --changes-option=<opt>  pass option <opt> to dpkg-genchanges.')
     . "\n\n" . g_(
@@ -150,6 +164,7 @@ sub usage {
 my $admindir;
 my @debian_rules = ('debian/rules');
 my @rootcommand = ();
+my $signbackend;
 my $signcommand;
 my $preclean = 1;
 my $postclean = 0;
@@ -162,7 +177,8 @@ my @source_opts;
 my $check_command = $ENV{DEB_CHECK_COMMAND};
 my @check_opts;
 my $signpause;
-my $signkey = $ENV{DEB_SIGN_KEYID};
+my $signkeyfile = $ENV{DEB_SIGN_KEYFILE};
+my $signkeyid = $ENV{DEB_SIGN_KEYID};
 my $signforce = 0;
 my $signreleased = 1;
 my $signsource = 1;
@@ -187,13 +203,33 @@ my @buildinfo_opts;
 my $changes_file;
 my @changes_opts;
 my %target_legacy_root = map { $_ => 1 } qw(
-    clean binary binary-arch binary-indep
+    clean
+    binary
+    binary-arch
+    binary-indep
 );
 my %target_official =  map { $_ => 1 } qw(
-    clean build build-arch build-indep binary binary-arch binary-indep
+    clean
+    build
+    build-arch
+    build-indep
+    binary
+    binary-arch
+    binary-indep
 );
 my @hook_names = qw(
-    init preclean source build binary buildinfo changes postclean check sign done
+    preinit
+    init
+    preclean
+    source
+    build
+    binary
+    buildinfo
+    changes
+    postclean
+    check
+    sign
+    done
 );
 my %hook;
 $hook{$_} = undef foreach @hook_names;
@@ -254,12 +290,15 @@ while (@ARGV) {
         } else {
             push @changes_opts, $changes_opt;
         }
-    } elsif (/^(?:-j|--jobs=)(\d*|auto)$/) {
-	$parallel = $1 || '';
-	$parallel_force = 1;
-    } elsif (/^(?:-J|--jobs-try=)(\d*|auto)$/) {
+    } elsif (/^--jobs(?:-try)?$/) {
+	$parallel = '';
+	$parallel_force = 0;
+    } elsif (/^(?:-[jJ]|--jobs(?:-try)?=)(\d*|auto)$/) {
 	$parallel = $1 || '';
 	$parallel_force = 0;
+    } elsif (/^--jobs-force(?:=(\d*|auto))?$/) {
+        $parallel = $1 || '';
+        $parallel_force = 1;
     } elsif (/^(?:-r|--root-command=)(.*)$/) {
 	my $arg = $1;
 	@rootcommand = split ' ', $arg;
@@ -274,13 +313,17 @@ while (@ARGV) {
 	usageerr(g_('missing hook %s command'), $hook_name)
 	    if not defined $hook_cmd;
 	$hook{$hook_name} = $hook_cmd;
-    } elsif (/^--buildinfo-id=.*$/) {
+    } elsif (/^(--buildinfo-id)=.*$/) {
 	# Deprecated option
-	warning('--buildinfo-id is deprecated, it is without effect');
+	warning(g_('%s is deprecated; it is without effect'), $1);
+    } elsif (/^--sign-backend=(.*)$/) {
+	$signbackend = $1;
     } elsif (/^(?:-p|--sign-command=)(.*)$/) {
 	$signcommand = $1;
-    } elsif (/^(?:-k|--sign-key=)(.*)$/) {
-	$signkey = $1;
+    } elsif (/^--sign-keyfile=(.*)$/) {
+	$signkeyfile = $1;
+    } elsif (/^(?:-k|--sign-keyid=|--sign-key=)(.*)$/) {
+	$signkeyid = $1;
     } elsif (/^--(no-)?check-builddeps$/) {
 	$checkbuilddep = !(defined $1 and $1 eq 'no-');
     } elsif (/^-([dD])$/) {
@@ -370,15 +413,15 @@ while (@ARGV) {
 	set_build_type(BUILD_FULL, $_);
     } elsif (/^-v(.*)$/) {
 	$since = $1;
-    } elsif (/^-m(.*)$/ or /^--release-by=(.*)$/) {
+    } elsif (/^-m(.*)$/ or /^--(?:source|build)-by=(.*)$/) {
 	$maint = $1;
-    } elsif (/^-e(.*)$/ or /^--build-by=(.*)$/) {
+    } elsif (/^-e(.*)$/ or /^--(?:changed|release)-by=(.*)$/) {
 	$changedby = $1;
     } elsif (/^-C(.*)$/) {
 	$desc = $1;
     } elsif (m/^-[EW]$/) {
 	# Deprecated option
-	warning(g_('-E and -W are deprecated, they are without effect'));
+	warning(g_('%s is deprecated; it is without effect'), $_);
     } elsif (/^-R(.*)$/ or /^--rules-file=(.*)$/) {
 	my $arg = $1;
 	@debian_rules = split ' ', $arg;
@@ -419,15 +462,8 @@ if ($check_command and not find_command($check_command)) {
     error(g_("check-command '%s' not found"), $check_command);
 }
 
-if ($signcommand) {
-    if (!find_command($signcommand)) {
-        error(g_("sign-command '%s' not found"), $signcommand);
-    }
-} elsif (($ENV{GNUPGHOME} && -e $ENV{GNUPGHOME}) ||
-         ($ENV{HOME} && -e "$ENV{HOME}/.gnupg")) {
-    if (find_command('gpg')) {
-        $signcommand = 'gpg';
-    }
+if ($signcommand and not find_command($signcommand)) {
+    error(g_("sign-command '%s' not found"), $signcommand);
 }
 
 # Default to auto if none of parallel=N, -J or -j have been specified.
@@ -435,6 +471,12 @@ if (not defined $parallel and not $build_opts->has('parallel')) {
     $parallel = 'auto';
 }
 
+#
+# Prepare the environment.
+#
+
+run_hook('preinit');
+
 if (defined $parallel) {
     if ($parallel eq 'auto') {
         # Most Unices.
@@ -458,7 +500,7 @@ if (defined $parallel) {
 
 if ($build_opts->has('terse')) {
     $ENV{MAKEFLAGS} //= '';
-    $ENV{MAKEFLAGS} .= ' -s';
+    $ENV{MAKEFLAGS} .= ' --no-print-directory';
 }
 
 set_build_profiles(@build_profiles);
@@ -468,7 +510,7 @@ my $ctrl = Dpkg::Control::Info->new();
 
 # Check whether we are doing some kind of rootless build, and sanity check
 # the fields values.
-my %rules_requires_root = parse_rules_requires_root($ctrl->get_source());
+my %rules_requires_root = parse_rules_requires_root($ctrl);
 
 my $pkg = mustsetvar($changelog->{source}, g_('source package'));
 my $version = mustsetvar($changelog->{version}, g_('source version'));
@@ -520,9 +562,33 @@ if (build_has_any(BUILD_ARCH_DEP)) {
 my $pv = "${pkg}_$sversion";
 my $pva = "${pkg}_${sversion}_$arch";
 
+my $signkeytype;
+my $signkeyhandle;
+if (defined $signkeyfile) {
+    $signkeytype = 'keyfile';
+    $signkeyhandle = bsd_glob($signkeyfile, GLOB_TILDE | GLOB_NOCHECK);
+} elsif (defined $signkeyid) {
+    $signkeytype = 'autoid';
+    $signkeyhandle = $signkeyid;
+} else {
+    $signkeytype = 'userid';
+    $signkeyhandle = $maintainer;
+}
+my $signkey = Dpkg::OpenPGP::KeyHandle->new(
+    type => $signkeytype,
+    handle => $signkeyhandle,
+);
 signkey_validate();
 
-if (not $signcommand) {
+my $openpgp = Dpkg::OpenPGP->new(
+    backend => $signbackend // 'auto',
+    cmd => $signcommand // 'auto',
+    needs => {
+        keystore => $signkey->needs_keystore(),
+    },
+);
+
+if (not $openpgp->can_use_secrets($signkey)) {
     $signsource = 0;
     $signbuildinfo = 0;
     $signchanges = 0;
@@ -551,7 +617,7 @@ if ($sanitize_env) {
 # Preparation of environment stops here
 #
 
-run_hook('init', 1);
+run_hook('init');
 
 if (not -x 'debian/rules') {
     warning(g_('debian/rules is not executable; fixing that'));
@@ -585,13 +651,20 @@ foreach my $call_target (@call_target) {
 }
 exit 0 if scalar @call_target;
 
-run_hook('preclean', $preclean);
+run_hook('preclean', {
+    enabled => $preclean,
+});
 
 if ($preclean) {
     run_rules_cond_root('clean');
 }
 
-run_hook('source', build_has_any(BUILD_SOURCE));
+run_hook('source', {
+    enabled => build_has_any(BUILD_SOURCE),
+    env => {
+        DPKG_BUILDPACKAGE_HOOK_SOURCE_OPTIONS => join(' ', @source_opts),
+    },
+});
 
 if (build_has_any(BUILD_SOURCE)) {
     warning(g_('building a source package without cleaning up as you asked; ' .
@@ -599,8 +672,6 @@ if (build_has_any(BUILD_SOURCE)) {
     run_cmd('dpkg-source', @source_opts, '-b', '.');
 }
 
-run_hook('build', build_has_any(BUILD_BINARY));
-
 my $build_types = get_build_options_from_type();
 
 if (build_has_any(BUILD_BINARY)) {
@@ -608,15 +679,31 @@ if (build_has_any(BUILD_BINARY)) {
     # targets. This is a temporary measure to not break too many packages
     # on a flag day.
     build_target_fallback($ctrl);
-
-    # If we are building rootless, there is no need to call the build target
-    # independently as non-root.
-    run_cmd(@debian_rules, $buildtarget) if rules_requires_root($binarytarget);
-    run_hook('binary', 1);
-    run_rules_cond_root($binarytarget);
 }
 
-run_hook('buildinfo', 1);
+# If we are building rootless, there is no need to call the build target
+# independently as non-root.
+if (build_has_any(BUILD_BINARY) && rules_requires_root($binarytarget)) {
+    run_hook('build', {
+        env => {
+            DPKG_BUILDPACKAGE_HOOK_BUILD_TARGET => $buildtarget,
+        },
+    });
+    run_cmd(@debian_rules, $buildtarget);
+} else {
+    run_hook('build', {
+        enabled => 0,
+    });
+}
+
+if (build_has_any(BUILD_BINARY)) {
+    run_hook('binary', {
+        env => {
+            DPKG_BUILDPACKAGE_HOOK_BINARY_TARGET => $binarytarget,
+        },
+    });
+    run_rules_cond_root($binarytarget);
+}
 
 $buildinfo_file //= "../$pva.buildinfo";
 
@@ -624,10 +711,13 @@ push @buildinfo_opts, "--build=$build_types" if build_has_none(BUILD_DEFAULT);
 push @buildinfo_opts, "--admindir=$admindir" if $admindir;
 push @buildinfo_opts, "-O$buildinfo_file" if $buildinfo_file;
 
+run_hook('buildinfo', {
+    env => {
+        DPKG_BUILDPACKAGE_HOOK_BUILDINFO_OPTIONS => join(' ', @buildinfo_opts),
+    },
+});
 run_cmd('dpkg-genbuildinfo', @buildinfo_opts);
 
-run_hook('changes', 1);
-
 $changes_file //= "../$pva.changes";
 
 push @changes_opts, "--build=$build_types" if build_has_none(BUILD_DEFAULT);
@@ -639,10 +729,17 @@ push @changes_opts, "-O$changes_file";
 
 my $changes = Dpkg::Control->new(type => CTRL_FILE_CHANGES);
 
+run_hook('changes', {
+    env => {
+        DPKG_BUILDPACKAGE_HOOK_CHANGES_OPTIONS => join(' ', @changes_opts),
+    },
+});
 run_cmd('dpkg-genchanges', @changes_opts);
 $changes->load($changes_file);
 
-run_hook('postclean', $postclean);
+run_hook('postclean', {
+    enabled => $postclean,
+});
 
 if ($postclean) {
     run_rules_cond_root('clean');
@@ -652,7 +749,12 @@ run_cmd('dpkg-source', @source_opts, '--after-build', '.');
 
 info(describe_build($changes->{'Files'}));
 
-run_hook('check', $check_command);
+run_hook('check', {
+    enabled => $check_command,
+    env => {
+        DPKG_BUILDPACKAGE_HOOK_CHECK_OPTIONS => join(' ', @check_opts),
+    },
+});
 
 if ($check_command) {
     run_cmd($check_command, @check_opts, $changes_file);
@@ -663,12 +765,12 @@ if ($signpause && ($signsource || $signbuildinfo || $signchanges)) {
     getc();
 }
 
-run_hook('sign', $signsource || $signbuildinfo || $signchanges);
+run_hook('sign', {
+    enabled => $signsource || $signbuildinfo || $signchanges,
+});
 
 if ($signsource) {
-    if (signfile("$pv.dsc")) {
-        error(g_('failed to sign %s file'), '.dsc');
-    }
+    signfile("$pv.dsc");
 
     # Recompute the checksums as the .dsc has changed now.
     my $buildinfo = Dpkg::Control->new(type => CTRL_FILE_BUILDINFO);
@@ -679,8 +781,8 @@ if ($signsource) {
     $checksums->export_to_control($buildinfo);
     $buildinfo->save($buildinfo_file);
 }
-if ($signbuildinfo && signfile("$pva.buildinfo")) {
-    error(g_('failed to sign %s file'), '.buildinfo');
+if ($signbuildinfo) {
+    signfile("$pva.buildinfo");
 }
 if ($signsource or $signbuildinfo) {
     # Recompute the checksums as the .dsc and/or .buildinfo have changed.
@@ -696,15 +798,15 @@ if ($signsource or $signbuildinfo) {
     update_files_field($changes, $checksums, "$pva.buildinfo");
     $changes->save($changes_file);
 }
-if ($signchanges && signfile("$pva.changes")) {
-    error(g_('failed to sign %s file'), '.changes');
+if ($signchanges) {
+    signfile("$pva.changes");
 }
 
 if (not $signreleased) {
     warning(g_('not signing UNRELEASED build; use --force-sign to override'));
 }
 
-run_hook('done', 1);
+run_hook('done');
 
 sub mustsetvar {
     my ($var, $text) = @_;
@@ -739,17 +841,25 @@ sub parse_rules_requires_root {
 
     my %rrr;
     my $rrr;
+    my $rrr_default;
     my $keywords_base;
     my $keywords_impl;
 
-    $rrr = $rrr_override // $ctrl->{'Rules-Requires-Root'} // 'binary-targets';
+    if (get_build_api($ctrl) >= 1) {
+        $rrr_default = 'no';
+    } else {
+        $rrr_default = 'binary-targets';
+    }
+
+    my $ctrl_src = $ctrl->get_source();
+    $rrr = $rrr_override // $ctrl_src->{'Rules-Requires-Root'} // $rrr_default;
 
     foreach my $keyword (split ' ', $rrr) {
         if ($keyword =~ m{/}) {
             if ($keyword =~ m{^dpkg/target/(.*)$}p and $target_official{$1}) {
                 error(g_('disallowed target in %s field keyword "%s"'),
                       'Rules-Requires-Root', $keyword);
-            } elsif ($keyword ne 'dpkg/target-subcommand') {
+            } elsif ($keyword =~ m{^dpkg/(.*)$} and $1 ne 'target-subcommand') {
                 error(g_('%s field keyword "%s" is unknown in dpkg namespace'),
                       'Rules-Requires-Root', $keyword);
             }
@@ -797,8 +907,10 @@ sub parse_rules_requires_root {
 }
 
 sub run_cmd {
-    printcmd(@_);
-    system @_ and subprocerr("@_");
+    my @cmd = @_;
+
+    printcmd(@cmd);
+    system @cmd and subprocerr("@cmd");
 }
 
 sub rules_requires_root {
@@ -821,8 +933,9 @@ sub run_rules_cond_root {
 }
 
 sub run_hook {
-    my ($name, $enabled) = @_;
+    my ($name, $opts) = @_;
     my $cmd = $hook{$name};
+    $opts->{enabled} //= 1;
 
     return if not $cmd;
 
@@ -830,11 +943,11 @@ sub run_hook {
 
     my %hook_vars = (
         '%' => '%',
-        'a' => $enabled ? 1 : 0,
-        'p' => $pkg,
-        'v' => $version,
-        's' => $sversion,
-        'u' => $uversion,
+        'a' => $opts->{enabled} ? 1 : 0,
+        'p' => $pkg // q{},
+        'v' => $version // q{},
+        's' => $sversion // q{},
+        'u' => $uversion // q{},
     );
 
     my $subst_hook_var = sub {
@@ -850,6 +963,11 @@ sub run_hook {
 
     $cmd =~ s/\%(.)/$subst_hook_var->($1)/eg;
 
+    $opts->{env}{DPKG_BUILDPACKAGE_HOOK_NAME} = $name;
+
+    # Set any environment variables for this hook invocation.
+    local @ENV{keys %{$opts->{env}}} = values %{$opts->{env}};
+
     run_cmd($cmd);
 }
 
@@ -865,17 +983,13 @@ sub update_files_field {
 }
 
 sub signkey_validate {
-    return unless defined $signkey;
-    # Make sure this is an hex keyid.
-    return unless $signkey =~ m/^(?:0x)?([[:xdigit:]]+)$/;
+    return unless $signkey->type eq 'keyid';
 
-    my $keyid = $1;
-
-    if (length $keyid <= 8) {
+    if (length $signkey->handle <= 8) {
         error(g_('short OpenPGP key IDs are broken; ' .
                  'please use key fingerprints in %s or %s instead'),
               '-k', 'DEB_SIGN_KEYID');
-    } elsif (length $keyid <= 16) {
+    } elsif (length $signkey->handle <= 16) {
         warning(g_('long OpenPGP key IDs are strongly discouraged; ' .
                    'please use key fingerprints in %s or %s instead'),
                 '-k', 'DEB_SIGN_KEYID');
@@ -896,17 +1010,15 @@ sub signfile {
     print { $signfh } "\n";
     close $signfh or syserr(g_('cannot close %s'), $signfile);
 
-    system($signcommand, '--utf8-strings', '--textmode', '--armor',
-           '--local-user', $signkey || $maintainer, '--clearsign',
-           '--weak-digest', 'SHA1', '--weak-digest', 'RIPEMD160',
-           '--output', "$signfile.asc", $signfile);
-    my $status = $?;
-    if ($status == 0) {
+    my $status = $openpgp->inline_sign($signfile, "$signfile.asc", $signkey);
+    if ($status == OPENPGP_OK) {
         move("$signfile.asc", "../$file")
             or syserror(g_('cannot move %s to %s'), "$signfile.asc", "../$file");
+    } else {
+        error(g_('failed to sign %s file: %s'), $file,
+              openpgp_errorcode_to_string($status));
     }
 
-    print "\n";
     return $status
 }
 
@@ -952,6 +1064,9 @@ sub build_target_fallback {
     return if $buildtarget eq 'build';
     return if scalar @debian_rules != 1;
 
+    # Avoid further heuristics in newer dpkg-build-api levels.
+    return if get_build_api($ctrl) >= 1;
+
     # Check if we are building both arch:all and arch:any packages, in which
     # case we now require working build-indep and build-arch targets.
     my $pkg_arch = 0;
