@@ -27,6 +27,7 @@ use Dpkg::Changelog::Debian;
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
 use Dpkg::Version;
+use Dpkg::Vendor qw(run_vendor_hook);
 
 textdomain('dpkg-dev');
 
@@ -39,7 +40,6 @@ sub join_lines($);
 
 BEGIN {
     eval q{
-        pop @INC if $INC[-1] eq '.';
         use Algorithm::Merge qw(merge);
     };
     if ($@) {
@@ -89,6 +89,8 @@ my @options_spec = (
     GetOptions(@options_spec);
 }
 
+my $backport_version_regex = run_vendor_hook('backport-version-regex');
+
 my ($old, $new_a, $new_b, $out_file) = @ARGV;
 unless (defined $old and defined $new_a and defined $new_b)
 {
@@ -114,13 +116,14 @@ my @b = reverse @$chb;
 my @result; # Lines to output
 my $exitcode = 0; # 1 if conflict encountered
 
-unless (merge_block($cho, $cha, $chb, sub {
-			my $changes = shift;
-			my $tail = $changes->get_unparsed_tail();
-			chomp $tail if defined $tail;
-			return $tail;
-		    }))
-{
+sub merge_tail {
+    my $changes = shift;
+    my $tail = $changes->get_unparsed_tail();
+    chomp $tail if defined $tail;
+    return $tail;
+};
+
+unless (merge_block($cho, $cha, $chb, \&merge_tail)) {
     merge_conflict($cha->get_unparsed_tail(), $chb->get_unparsed_tail());
 }
 
@@ -196,8 +199,10 @@ sub compare_versions {
                     $b->get_distributions() eq 'UNRELEASED';
     }
     # Backports are not real prereleases.
-    $av =~ s/~(bpo|deb)/+$1/;
-    $bv =~ s/~(bpo|deb)/+$1/;
+    if (defined $backport_version_regex) {
+        $a =~ s/$backport_version_regex/+$1/;
+        $b =~ s/$backport_version_regex/+$1/;
+    }
     if ($merge_prereleases) {
         $av =~ s/~[^~]*$//;
         $bv =~ s/~[^~]*$//;
